using System;
using System.Collections.Generic;
using System.Text;

namespace Vevo.Shared.Utilities
{
    public class TokenProcessor
    {
        private string _text;
        private int _currentIndex;

        public enum CharType
        {
            NonLetterDigit
        }


        public TokenProcessor( string text )
        {
            _text = text;
            _currentIndex = 0;
        }


        private bool IsCharType( char value, CharType type )
        {
            bool result = false;

            switch (type)
            {
                case CharType.NonLetterDigit:
                    if (!Char.IsLetterOrDigit( value ))
                        result = true;
                    break;
            }

            return result;
        }

        public bool IsEndOfString()
        {
            if (_text.Length > 0)
                return (_currentIndex >= _text.Length);
            else
                return true;
        }

        // Move to, but not passing, marker
        public void SkipTo( char marker )
        {
            _currentIndex = _text.IndexOf( marker, _currentIndex );

            if (_currentIndex == -1)
                _currentIndex = _text.Length;
        }

        // Extract to, but not including, end marker. If no marker is found, extract to the end of string
        public string ExtractTo( CharType endMarkerType )
        {
            int endIndex = _currentIndex;
            bool found = false;

            while (!found && endIndex < _text.Length)
            {
                if (IsCharType( _text[endIndex], endMarkerType ))
                    found = true;

                endIndex++;
            }

            if (found)
                endIndex--;

            string result = _text.Substring( _currentIndex, endIndex - _currentIndex );
            _currentIndex = endIndex;

            return result;
        }

        public void SeekRelative( int distance )
        {
            _currentIndex += distance;

            if (_currentIndex < 0)
                _currentIndex = 0;
            else if (_currentIndex > _text.Length)
                _currentIndex = _text.Length;
        }

        public char GetRelativeChar( int index )
        {
            return _text[_currentIndex + index];
        }

    }
}
