﻿using System;
using System.Collections.Generic;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.EmailTemplates;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class EmailTemplateDetailRepository : IEmailTemplateDetailRepository
    {
        #region Private

        private const string RepositoryName = "EmailTemplateRepository";
        private const string GetLocalesSignature = "GetLocales";
        private const string GetOneSignature = "GetOne";
        private const string GetIDByNameAndStoreIDSignature = "GetIDByNameAndStoreID";
        private const string GetAllSignature = "GetAll";
        private const string GetTableSchemaSignature = "GetTableSchema";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility(
            new string[] { "EmailTemplateDetailLocale", "EmailTemplateDetail" } );
        private EmailTemplateDetailRepositoryNoCache _repository = new EmailTemplateDetailRepositoryNoCache();

        #endregion

        #region Constructors

        public EmailTemplateDetailRepository()
        {
        }

        public EmailTemplateDetailRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public IList<ILocale> GetLocales( string emailTemplateID )
        {
            IList<ILocale> list = (IList<ILocale>) _dataCache.GetValue( GetLocalesSignature, _cacheDependency, emailTemplateID );

            if (list == null)
            {
                list = _repository.GetLocales( emailTemplateID );
                _dataCache.SetValue( GetLocalesSignature, list, emailTemplateID );
            }

            return list;
        }

        public EmailTemplateDetail Save( EmailTemplateDetail emailTemplateDetail )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            EmailTemplateDetail newEmailTemplateDetail = _repository.Save( emailTemplateDetail );
            _dataCache.ClearRepositoryCache();
            return newEmailTemplateDetail;
        }

        public EmailTemplateDetail GetOne( Culture culture, string emailTemplateDetailID )
        {
            EmailTemplateDetail result = (EmailTemplateDetail) _dataCache.GetValue(
                GetOneSignature, _cacheDependency, culture.CultureID, emailTemplateDetailID );

            if (result == null)
            {
                result = _repository.GetOne( culture, emailTemplateDetailID );
                _dataCache.SetValue( GetOneSignature, result, culture.CultureID, emailTemplateDetailID );
            }

            return result;
        }

        public string GetIDByNameAndStoreID( Culture culture, string emailTemplateDetailName, string storeID )
        {
            object result = _dataCache.GetValue( GetIDByNameAndStoreIDSignature, _cacheDependency, culture.CultureID, emailTemplateDetailName, storeID );

            if (result == null)
            {
                result = _repository.GetIDByNameAndStoreID( culture, emailTemplateDetailName, storeID );
                _dataCache.SetValue( GetIDByNameAndStoreIDSignature, result, culture.CultureID, emailTemplateDetailName, storeID );
            }

            return result.ToString();
        }

        public void Delete( string emailTemplateDetailID )
        {
            _repository.Delete( emailTemplateDetailID );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            _repository.DeleteLocalesByCultureID( cultureID );
            _dataCache.ClearRepositoryCache();
        }

        public IList<EmailTemplateDetail> GetAll( Culture culture, string sortBy, string storeID )
        {
            IList<EmailTemplateDetail> list = (IList<EmailTemplateDetail>) _dataCache.GetValue(
                GetAllSignature, _cacheDependency, culture.CultureID, sortBy, storeID );

            if (list == null)
            {
                list = _repository.GetAll( culture, sortBy, storeID );
                _dataCache.SetValue( GetAllSignature, list, culture.CultureID, sortBy, storeID );
            }

            return list;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue( GetTableSchemaSignature, _cacheDependency );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue( GetTableSchemaSignature, list );
            }

            return list;
        }

        #endregion
    }
}
