﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Data;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Domain.GoogleFeed;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data.Implementation
{
    internal class GoogleOptionMappingAccess
    {
        #region Private

        private GoogleOptionMapping CreateOneFromRow( DataRow row )
        {
            GoogleOptionMapping optionMapping = new GoogleOptionMapping();
            optionMapping.GoogleFeedTagID = row["GoogleFeedTagID"].ToString();
            optionMapping.GoogleFeedTagName = row["TagName"].ToString();
            optionMapping.OptionGroupID = row["OptionGroupID"].ToString();
            optionMapping.OptionGroupName = row["Name"].ToString();
            return optionMapping;
        }

        private IList<GoogleOptionMapping> ConvertTableToList( DataTable table )
        {
            IList<GoogleOptionMapping> list = new List<GoogleOptionMapping>();

            foreach (DataRow row in table.Rows)
                list.Add( CreateOneFromRow( row ) );

            return list;
        }

        private IList<GoogleOptionMapping> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<GoogleOptionMapping> list = new List<GoogleOptionMapping>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        private GoogleOptionMapping Create( Culture culture, GoogleOptionMapping googleOptionMapping )
        {
            int rowAffect = DataAccess.ExecuteNonQuery(
                "INSERT INTO GoogleOptionMapping (GoogleFeedTagID ,OptionGroupID ) " +
                "VALUES ( @GoogleFeedTagID, @OptionGroupID );",
                DataAccess.CreateParameterString( googleOptionMapping.GoogleFeedTagID ),
                DataAccess.CreateParameterString( googleOptionMapping.OptionGroupID ) );

            if (rowAffect == 1)
                return GetOne( culture, googleOptionMapping.GoogleFeedTagID, googleOptionMapping.OptionGroupID );
            else
                return GoogleOptionMapping.Null;
        }

        private int GetGoogleOptionMappingCount( Culture culture )
        {
            string count = DataAccess.ExecuteScalar(
                "SELECT COUNT (GoogleOptionMapping.GoogleFeedTagID) AS [RowCount] " +
                "FROM GoogleOptionMapping " +
                "LEFT JOIN " +
                "    (SELECT GoogleFeedTagID, TagName FROM GoogleFeedTag) AS GoogleFeedTag " +
                "     ON GoogleOptionMapping.GoogleFeedTagID = GoogleFeedTag.GoogleFeedTagID " +
                "LEFT JOIN " +
                "    (SELECT OptionGroup.OptionGroupID , GroupLocale.Name, GroupLocale.DisplayText FROM OptionGroup " +
                "     LEFT JOIN " +
                "        (SELECT OptionGroupID, Name, DisplayText FROM OptionGroupLocale WHERE CultureID = @CultureID) AS GroupLocale " +
                "         ON OptionGroup.OptionGroupID = GroupLocale.OptionGroupID) AS OptionGroup " +
                "ON GoogleOptionMapping.OptionGroupID = OptionGroup.OptionGroupID ",
                DataAccess.CreateParameterString( culture.CultureID ) );
            return ConvertUtilities.ToInt32( count );
        }

        private string CreateSortByText( string sortBy )
        {
            if (String.IsNullOrEmpty( sortBy ))
                return " ORDER BY GoogleOptionMapping.GoogleFeedTagID ";
            else if (sortBy.Trim().ToLower().StartsWith( "GoogleFeedTagID" ))
                return " ORDER BY GoogleOptionMapping." + sortBy.Trim() + " ";
            else
                return " ORDER BY " + sortBy + " ";
        }

        private string ConvertFieldName( string fieldName )
        {
            switch (fieldName.ToLower())
            {
                case "GoogleFeedTagID":
                    return "GoogleOptionMapping.GoogleFeedTagID";
                case "OptionGroupID":
                    return "GoogleOptionMapping.OptionGroupID";
                default:
                    return fieldName;
            }
        }

        private string DeleteCondition( string googleFeedTagID, string optionGroupID )
        {
            if (String.IsNullOrEmpty( googleFeedTagID ))
                return " WHERE OptionGroupID = @OptionGroupID ";
            else if (String.IsNullOrEmpty( optionGroupID ))
                return " WHERE GoogleFeedTagID = @GoogleFeedTagID ";
            else
                return " WHERE GoogleFeedTagID = @GoogleFeedTagID AND OptionGroupID = @OptionGroupID ";
        }

        #endregion

        #region Public

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT GoogleOptionMapping.GoogleFeedTagID, GoogleOptionMapping.OptionGroupID, " +
                "GoogleFeedTag.TagName, OptionGroup.Name " +
                "FROM GoogleOptionMapping " +
                "LEFT JOIN " +
                "    (SELECT GoogleFeedTagID, TagName FROM GoogleFeedTag) AS GoogleFeedTag " +
                "     ON GoogleOptionMapping.GoogleFeedTagID = GoogleFeedTag.GoogleFeedTagID " +
                "LEFT JOIN " +
                "    (SELECT OptionGroup.OptionGroupID , GroupLocale.Name, GroupLocale.DisplayText FROM OptionGroup " +
                "     LEFT JOIN " +
                "        (SELECT OptionGroupID, Name, DisplayText FROM OptionGroupLocale) AS GroupLocale " +
                "         ON OptionGroup.OptionGroupID = GroupLocale.OptionGroupID) AS OptionGroup " +
                "ON GoogleOptionMapping.OptionGroupID = OptionGroup.OptionGroupID " +
                "WHERE 1 = 0 " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public void Delete( string optionGroupID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM GoogleOptionMapping " +
                DeleteCondition( String.Empty, optionGroupID ),
                DataAccess.CreateParameterString( optionGroupID ) );
        }

        public void Delete( string googleFeedTagID, string optionGroupID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM GoogleOptionMapping " +
                DeleteCondition( googleFeedTagID, optionGroupID ),
                DataAccess.CreateParameterString( googleFeedTagID ),
                DataAccess.CreateParameterString( optionGroupID ) );
        }

        public GoogleOptionMapping Save( Culture culture, GoogleOptionMapping googleOptionMapping )
        {
            GoogleOptionMapping result = GetOne( culture, googleOptionMapping.GoogleFeedTagID, googleOptionMapping.OptionGroupID );
            if (result.IsNull)
                return Create( culture, googleOptionMapping );
            else
                return result;
        }

        public GoogleOptionMapping Update( Culture culture, string oldGoogleFeedTagID, string oldOptionGroupID, GoogleOptionMapping newGoogleOptionMapping )
        {
            Delete( oldGoogleFeedTagID, oldOptionGroupID );
            return Create( culture, newGoogleOptionMapping );
        }

        public GoogleOptionMapping GetOne( Culture culture, string googleFeedTagID, string optionGroupID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT GoogleOptionMapping.GoogleFeedTagID, GoogleOptionMapping.OptionGroupID, " +
                "GoogleFeedTag.TagName, OptionGroup.Name " +
                "FROM GoogleOptionMapping " +
                "LEFT JOIN " +
                "    (SELECT GoogleFeedTagID, TagName FROM GoogleFeedTag) AS GoogleFeedTag " +
                "     ON GoogleOptionMapping.GoogleFeedTagID = GoogleFeedTag.GoogleFeedTagID " +
                "LEFT JOIN " +
                "    (SELECT OptionGroup.OptionGroupID , GroupLocale.Name, GroupLocale.DisplayText FROM OptionGroup " +
                "     LEFT JOIN " +
                "        (SELECT OptionGroupID, Name, DisplayText FROM OptionGroupLocale WHERE CultureID = @CultureID) AS GroupLocale " +
                "         ON OptionGroup.OptionGroupID = GroupLocale.OptionGroupID) AS OptionGroup " +
                "ON GoogleOptionMapping.OptionGroupID = OptionGroup.OptionGroupID " +
                "WHERE GoogleOptionMapping.GoogleFeedTagID = @GoogleFeedTagID AND GoogleOptionMapping.OptionGroupID = @OptionGroupID ",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( googleFeedTagID ),
                DataAccess.CreateParameterString( optionGroupID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return GoogleOptionMapping.Null;
        }

        public IList<GoogleOptionMapping> GetAll( Culture culture )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT GoogleOptionMapping.GoogleFeedTagID, GoogleOptionMapping.OptionGroupID, " +
                "GoogleFeedTag.TagName, OptionGroup.Name " +
                "FROM GoogleOptionMapping " +
                "LEFT JOIN " +
                "    (SELECT GoogleFeedTagID, TagName FROM GoogleFeedTag) AS GoogleFeedTag " +
                "     ON GoogleOptionMapping.GoogleFeedTagID = GoogleFeedTag.GoogleFeedTagID " +
                "LEFT JOIN " +
                "    (SELECT OptionGroup.OptionGroupID , GroupLocale.Name, GroupLocale.DisplayText FROM OptionGroup " +
                "     LEFT JOIN " +
                "        (SELECT OptionGroupID, Name, DisplayText FROM OptionGroupLocale WHERE CultureID = @CultureID) AS GroupLocale " +
                "         ON OptionGroup.OptionGroupID = GroupLocale.OptionGroupID) AS OptionGroup " +
                "ON GoogleOptionMapping.OptionGroupID = OptionGroup.OptionGroupID ",
                DataAccess.CreateParameterString( culture.CultureID ) );

            return ConvertTableToList( table );
        }

        public IList<GoogleOptionMapping> SearchGoogleOptionMapping(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            ArrayList parameterList = new ArrayList();
            parameterList.Add( DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ) );
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string optionFieldName = ConvertFieldName( searchFilter.FieldName );
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, optionFieldName, parameterList );

            string query = "SELECT ROW_NUMBER() over ( " + CreateSortByText( sortBy ) + ") as rownum, " +
                " * FROM (" +
                "SELECT GoogleOptionMapping.GoogleFeedTagID, GoogleOptionMapping.OptionGroupID, " +
                "GoogleFeedTag.TagName, OptionGroup.Name " +
                "FROM GoogleOptionMapping " +
                "LEFT JOIN " +
                "    (SELECT GoogleFeedTagID, TagName FROM GoogleFeedTag) AS GoogleFeedTag " +
                "     ON GoogleOptionMapping.GoogleFeedTagID = GoogleFeedTag.GoogleFeedTagID " +
                "LEFT JOIN " +
                "    (SELECT OptionGroup.OptionGroupID , GroupLocale.Name, GroupLocale.DisplayText FROM OptionGroup " +
                "     LEFT JOIN " +
                "        (SELECT OptionGroupID, Name, DisplayText FROM OptionGroupLocale WHERE CultureID = @CultureID) AS GroupLocale " +
                "         ON OptionGroup.OptionGroupID = GroupLocale.OptionGroupID) AS OptionGroup " +
                "ON GoogleOptionMapping.OptionGroupID = OptionGroup.OptionGroupID " +
                commandSearch + " ) AS GoogleOptionMapping ";

            string extQuery = "with SearchResult as (" + query + ") " +
                    " select * from SearchResult where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            DataTable full = DataAccess.ExecuteSelect( extQuery,
                DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            howManyItems = GetGoogleOptionMappingCount( culture );

            return ConvertTableToList( full, startIndex, endIndex );
        }

        #endregion
    }
}
