using System;
using System.Collections.Generic;
using System.Text;
using Vevo.Domain.Base;

namespace Vevo.Domain.Products
{
    public class OptionItemDisplay
    {
        private string _priceUp;
        private OptionItem _optionItem;

        public OptionItemDisplay( OptionItem optionItem )
        {
            _optionItem = optionItem;
        }

        public string PriceUp { get { return _priceUp; } set { _priceUp = value; } }
        public string OptionItemID { get { return _optionItem.OptionItemID; } }
        public string Name { get { return _optionItem.Name; } }
    }

    [Serializable]
    public class OptionItem : ILocaleLoader, ICloneable
    {
        #region Enum

        public enum PriceType
        {
            Price,
            Percentage
        }

        #endregion


        #region Private

        private string _optionItemID = "0";
        private string _optionGroupID = "0";
        private PriceType _optionItemType;
        private decimal _priceToAdd = 0;
        private double _percentageChange = 0;
        private double _weightToAdd = 0;
        private string _imageFile = String.Empty;

        private Culture _defaultCulture;
        private LocaleCollection<OptionItemLocale> _locales;

        #endregion


        #region Public Constructors

        public OptionItem( Culture defaultCulture )
        {
            _defaultCulture = defaultCulture;
            _locales = new LocaleCollection<OptionItemLocale>( this );
        }

        #endregion


        #region Public Properties

        public static OptionItem Null
        {
            get { return NullOptionItem.Instance; }
        }

        public virtual bool IsNull
        {
            get { return false; }
        }

        public string OptionItemID { get { return _optionItemID; } set { _optionItemID = value; } }
        public string OptionGroupID { get { return _optionGroupID; } set { _optionGroupID = value; } }
        public PriceType OptionItemType { get { return _optionItemType; } set { _optionItemType = value; } }
        public decimal PriceToAdd { get { return _priceToAdd; } set { _priceToAdd = value; } }
        public double PercentageChange { get { return _percentageChange; } set { _percentageChange = value; } }
        public double WeightToAdd { get { return _weightToAdd; } set { _weightToAdd = value; } }
        public string ImageFile { get { return _imageFile; } set { _imageFile = value; } }

        public Culture DefaultCulture { get { return _defaultCulture; } set { _defaultCulture = value; } }
        public LocaleCollection<OptionItemLocale> Locales { get { return _locales; } }


        //------------------------------------------------------------
        // Locale items
        public string Name
        {
            get { return Locales[DefaultCulture].Name; }
            set { Locales.GetNoLoad( DefaultCulture ).Name = value; }
        }
        // End Locale items
        //------------------------------------------------------------

        #endregion


        #region ILocaleLoader Members

        public IList<ILocale> GetLocales()
        {
            List<ILocale> result = new List<ILocale>();

            foreach (OptionItemLocale locale in DataAccessContext.OptionItemRepository.GetLocales( OptionItemID ))
                result.Add( locale );

            return result;
        }

        #endregion


        #region IClonable Members

        // Copy the whole aggregate (including locale)
        public object Clone()
        {
            OptionItem newObject = (OptionItem) MemberwiseClone();
            newObject._locales = (LocaleCollection<OptionItemLocale>) Locales.Clone();

            return newObject;
        }

        #endregion


        #region Public Methods

        public decimal GetOptionPrice( decimal productPrice )
        {
            if (OptionItemType == PriceType.Price)
                return PriceToAdd;
            else
                return (decimal) (PercentageChange / 100) * productPrice;
        }

        #endregion

    }

}
