using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Data;
using System.Text;

using Vevo.Shared.Utilities;

namespace Vevo.Domain.Products
{
    public class ProductFactory
    {
        #region Private

        private Culture _defaultCulture;
        private Hashtable _baseData = new Hashtable();
        private Hashtable _giftCertificateData = new Hashtable();
        private IList<String> _parentCategoryIDs = new List<String>();
        private IList<String> _parentDepartmentIDs = new List<String>();

        private GiftCertificateProduct.ExpireTypeEnum ConvertToExpireType( object expireTypeString )
        {
            GiftCertificateProduct.ExpireTypeEnum expireType;
            try
            {
                expireType = EnumUtilities<GiftCertificateProduct.ExpireTypeEnum>.Parse(
                    ConvertUtilities.ToString( expireTypeString ) );
            }
            catch
            {
                expireType = GiftCertificateProduct.ExpireTypeEnum.None;
            }

            return expireType;
        }

        private Product CreateObject()
        {
            if (RequiresGiftCertificateData)
                return new GiftCertificateProduct( _defaultCulture );
            else
                return new Product( _defaultCulture );
        }

        private void PopulateBaseData( Product product, string storeID )
        {
            product.ProductID = ConvertUtilities.ToString( _baseData["ProductID"] );
            product.Name = ConvertUtilities.ToString( _baseData["Name"] );
            product.ShortDescription = ConvertUtilities.ToString( _baseData["ShortDescription"] );
            product.LongDescription = ConvertUtilities.ToString( _baseData["LongDescription"] );
            product.Sku = ConvertUtilities.ToString( _baseData["Sku"] );
            product.Model = ConvertUtilities.ToString( _baseData["Model"] );
            product.ImageSecondary = ConvertUtilities.ToString( _baseData["ImageSecondary"] );
            product.UseInventory = ConvertUtilities.ToBoolean( _baseData["UseInventory"] );
            product.Weight = ConvertUtilities.ToDouble( _baseData["Weight"] );
            product.FixedShippingCost = ConvertUtilities.ToBoolean( _baseData["FixedShippingCost"] );
            product.FreeShippingCost = ConvertUtilities.ToBoolean( _baseData["FreeShippingCost"] );
            product.ShippingCost = ConvertUtilities.ToDecimal( _baseData["ShippingCost"] );
            product.Manufacturer = ConvertUtilities.ToString( _baseData["Manufacturer"] );


            product.Keywords = ConvertUtilities.ToString( _baseData["Keywords"] );
            product.CategoryIDs = _parentCategoryIDs;
            product.DepartmentIDs = _parentDepartmentIDs;
            product.UrlName = ConvertUtilities.ToString( _baseData["UrlName"] );
            product.IsTodaySpecial = ConvertUtilities.ToBoolean( _baseData["IsTodaySpecial"] );
            product.MinQuantity = ConvertUtilities.ToInt32( _baseData["MinQuantity"] );
            product.MaxQuantity = ConvertUtilities.ToInt32( _baseData["MaxQuantity"] );
            product.IsDownloadable = ConvertUtilities.ToBoolean( _baseData["IsDownloadable"] );
            product.DownloadPath = ConvertUtilities.ToString( _baseData["DownloadPath"] );
            product.IsEnabled = ConvertUtilities.ToBoolean( _baseData["IsEnabled"] );
            product.RelatedProducts = ConvertUtilities.ToString( _baseData["RelatedProducts"] );
            product.MerchantRating = ConvertUtilities.ToDouble( _baseData["MerchantRating"] );

            product.IsGiftCertificate = ConvertUtilities.ToBoolean( _baseData["IsGiftCertificate"] );
            product.IsFixedPrice = ConvertUtilities.ToBoolean( _baseData["IsFixedPrice"] );

            if (ConvertUtilities.ToInt32( _baseData["ProductDiscountGroupID"] ) == 0)
            {
                if (ConvertUtilities.ToInt32( _baseData["CategoryDiscountGroupID"] ) == 0)
                    product.DiscountGroupID = "0";
                else
                    product.DiscountGroupID = ConvertUtilities.ToString( _baseData["CategoryDiscountGroupID"] );
            }
            else
            {
                product.DiscountGroupID = ConvertUtilities.ToString( _baseData["ProductDiscountGroupID"] );
            }

            product.ManufacturerPartNumber = ConvertUtilities.ToString( _baseData["ManufacturerPartNumber"] );
            product.Upc = ConvertUtilities.ToString( _baseData["Upc"] );
            product.TaxClassID = ConvertUtilities.ToString( _baseData["TaxClassID"] );
            product.IsParentVisible = ConvertUtilities.ToBoolean( _baseData["IsParentVisible"] );
            product.Other1 = ConvertUtilities.ToString( _baseData["Other1"] );
            product.Other2 = ConvertUtilities.ToString( _baseData["Other2"] );
            product.Other3 = ConvertUtilities.ToString( _baseData["Other3"] );
            product.Other4 = ConvertUtilities.ToString( _baseData["Other4"] );
            product.Other5 = ConvertUtilities.ToString( _baseData["Other5"] );
            product.ProductDetailsLayoutPath = ConvertUtilities.ToString( _baseData["ProductDetailsLayoutPath"] );

            product.IsRecurring = ConvertUtilities.ToBoolean( _baseData["IsRecurring"] );
            product.IsAffiliate = ConvertUtilities.ToBoolean( _baseData["IsAffiliate"] );
            product.SetMetaKeyword( storeID, ConvertUtilities.ToString( _baseData["MetaKeyword"] ) );
            product.SetMetaDescription( storeID, ConvertUtilities.ToString( _baseData["MetaDescription"] ) );
            product.QBListID = ConvertUtilities.ToString( _baseData["QBListID"] );
            product.IsQBExported = ConvertUtilities.ToBoolean( _baseData["IsQBExported"] );
            product.IsCustomPrice = ConvertUtilities.ToBoolean( _baseData["IsCustomPrice"] );
            product.IsCallForPrice = ConvertUtilities.ToBoolean( _baseData["IsCallForPrice"] );
            product.Brand = ConvertUtilities.ToString( _baseData["Brand"] );
            product.ReturnTime = ConvertUtilities.ToInt32( _baseData["ReturnTime"] );
            product.IsProductKit = ConvertUtilities.ToBoolean( _baseData["IsProductKit"] );
            product.IsDynamicProductKitPrice = ConvertUtilities.ToBoolean( _baseData["IsDynamicProductKitPrice"] );
            product.IsDynamicProductKitWeight = ConvertUtilities.ToBoolean( _baseData["IsDynamicProductKitWeight"] );
            product.CreateDate = ConvertUtilities.ToDateTime( _baseData["CreateDate"] );

            product.ViewCount = ConvertUtilities.ToInt32( _baseData["ViewCount"] );
        }

        private void PopulateGiftCertificateData( GiftCertificateProduct product )
        {
            product.GiftAmount = ConvertUtilities.ToDecimal( _giftCertificateData["GiftAmount"] );
            product.ExpireType = ConvertToExpireType( _giftCertificateData["ExpireType"] );
            product.FixedExpireDate = ConvertUtilities.ToDateTime( _giftCertificateData["FixedExpireDate"] );
            product.NumberOfDays = ConvertUtilities.ToInt32( _giftCertificateData["NumberOfDays"] );
            product.IsElectronic = ConvertUtilities.ToBoolean( _giftCertificateData["IsElectronic"] );
        }

        private void PopulateRecurringData( Product product )
        {
            ProductRecurring recurring = new ProductRecurring();
            recurring.RecurringInterval = ConvertUtilities.ToInt32( _baseData["RecurringInterval"] );
            recurring.RecurringIntervalUnit = ConvertUtilities.ToString( _baseData["RecurringIntervalUnit"] );
            recurring.RecurringNumberOfCycles = ConvertUtilities.ToInt32( _baseData["RecurringNumberOfCycles"] );
            recurring.RecurringNumberOfTrialCycles = ConvertUtilities.ToInt32(
                _baseData["RecurringNumberOfTrialCycles"] );
            recurring.RecurringTrialAmount = ConvertUtilities.ToDecimal( _baseData["RecurringTrialAmount"] );
            product.ProductRecurring = recurring;
        }

        private void PopulateCustomPrice( Product product )
        {
            ProductCustomPrice customPrice = new ProductCustomPrice();
            customPrice.DefaultPrice = ConvertUtilities.ToDecimal( _baseData["DefaultPrice"] );
            customPrice.MinimumPrice = ConvertUtilities.ToDecimal( _baseData["MinimumPrice"] );
            product.ProductCustomPrice = customPrice;
        }

        #endregion


        #region Public Properties

        public Culture DefaultCulture { get { return _defaultCulture; } set { _defaultCulture = value; } }
        public Hashtable BaseData { get { return _baseData; } set { _baseData = value; } }
        public Hashtable GiftCertificateData
        {
            get { return _giftCertificateData; }
            set { _giftCertificateData = value; }
        }

        public IList<String> ParentCategoryIDs
        {
            get { return _parentCategoryIDs; }
            set { _parentCategoryIDs = value; }
        }

        public IList<String> ParentDepartmentIDs
        {
            get { return _parentDepartmentIDs; }
            set { _parentDepartmentIDs = value; }
        }

        public bool RequiresGiftCertificateData
        {
            get
            {
                return ConvertUtilities.ToBoolean( BaseData["IsGiftCertificate"] );
            }
        }

        public bool RequiresRecurringData
        {
            get
            {
                return ConvertUtilities.ToBoolean( BaseData["IsRecurring"] );
            }
        }

        #endregion


        #region Public Members

        public Product CreateProduct( string storeID )
        {
            Product product = CreateObject();
            PopulateBaseData( product, storeID );

            if (product is GiftCertificateProduct)
            {
                PopulateGiftCertificateData( (GiftCertificateProduct) product );
            }

            if (product.IsRecurring)
            {
                PopulateRecurringData( product );
            }

            if (ProductCustomPrice.VerifyIsUseCustomPrice( product.ProductID ))
            {
                PopulateCustomPrice( product );
            }
            return product;
        }

        #endregion
    }
}
