using System;
using System.Data;
using System.Collections.Generic;
using System.Configuration;

using Vevo.Domain.Configurations;
using Vevo.Domain.Exceptions;
using Vevo.Domain.Orders;
using Vevo.Domain.Users;

namespace Vevo.Domain.Shipping.Custom
{
    /// <summary>
    /// Summary description for ByWeightShippingMethod
    /// </summary>
    public class ByWeightShippingMethod : CustomShippingMethod
    {
        #region Private

        private decimal GetByWeightShippingCost( CartItemGroup cartItemGroup )
        {
            CartItemGroup nonOverridden = new CartItemGroup( GetNonOverriddenShippingCartItems( cartItemGroup ) );
            double totalWeight = nonOverridden.GetTotalWeight();

            ShippingWeightRate weightRate = SelectApplicableWeightRate( totalWeight );
            return weightRate.WeightRate;
        }

        #endregion


        #region Constructors

        public ByWeightShippingMethod(
            ShippingOption shippingOption,
            ShippingChoice selectedShippingChoice )     // ignored in non-real-time shipping option

            : base( shippingOption )
        {
        }

        #endregion


        #region Public Constances

        public const string ShippingOptionTypeName = "ByWeight";

        #endregion


        #region Public Methods

        public override decimal GetShippingCost(
            CartItemGroup cartItemGroup,
            WholesaleStatus wholesaleStatus,
            decimal discount )
        {
            if (IsFreeShippingForNonOverriddenItems( cartItemGroup, wholesaleStatus ))
                return GetOverriddenShippingAmount( cartItemGroup );
            else
                return GetOverriddenShippingAmount( cartItemGroup )
                    + GetByWeightShippingCost( cartItemGroup );
        }

        public ShippingWeightRate SelectApplicableWeightRate( double weight )
        {
            IList<ShippingWeightRate> weightRates =
                DataAccessContext.ShippingWeightRateRepository.GetAllByShippingID(
                    ShippingOption.ShippingID,
                    "ToWeight" );

            for (int i = 0; i < weightRates.Count; i++)
            {
                if (weightRates[i].ToWeight >= weight)
                    return weightRates[i];
            }

            throw new ShippingException(
                "Weight is out of range. "
                + "Weight: " + weight + ", Maximum Weight: " + weightRates[weightRates.Count - 1].WeightRate
                + ", ShippingID: " + ShippingOption.ShippingID );
        }

        #endregion

    }
}
