using System;
using System.Collections;
using System.Data;
using System.IO;
using System.Text;
using Vevo.Shared.Utilities;

namespace Vevo.Reports.Exporters
{
    public class SaleReportExporter
    {
        private string _baseCurrencySymbol;
        private DataTable _table;
        public SaleReportExporter( string baseCurrencySymbol )
        {
            _baseCurrencySymbol = baseCurrencySymbol;
        }

        private string GenerateHeader()
        {
            return "\"Date/Time\",\"Order Total\",\"Number Of Order\"," +
                   "\"Average\",\"Number Of Items Sold\",\"Average Items Per Order\"";
        }

        private string GenerateDetail( DataRow row )
        {
            string period, orderTotal, numberOfOrder, averageOrderTotal, numberOfItemsSold, averageItemsPerOrder;
            period = row["Period"].ToString();
            orderTotal = _baseCurrencySymbol + row["Total"].ToString();
            numberOfOrder = row["NumberOfOrder"].ToString();
            averageOrderTotal = _baseCurrencySymbol + row["Average"].ToString();
            numberOfItemsSold = row["Quantity"].ToString();
            averageItemsPerOrder = row["AvgQuantity"].ToString();

            return period + "," + orderTotal + "," + numberOfOrder + "," + averageOrderTotal + "," +
                numberOfItemsSold + "," + averageItemsPerOrder;
        }
        private string GenerateFooter()
        {
            decimal total = 0;
            decimal numberOfOrder = 0;
            decimal quantity = 0;
            foreach (DataRow row in _table.Rows)
            {
                total += ConvertUtilities.ToDecimal( row["Total"].ToString() );
                numberOfOrder += ConvertUtilities.ToInt32( row["NumberOfOrder"].ToString() );
                quantity += ConvertUtilities.ToInt32( row["Quantity"].ToString() );
            }
            decimal Average = total / numberOfOrder;
            decimal AverageItemsPerOrder = quantity / numberOfOrder;

            return "\"Total\"" + "," + _baseCurrencySymbol + total + "," + numberOfOrder + "," + _baseCurrencySymbol + (Average) + "," + quantity + "," + (AverageItemsPerOrder);
        }
        public string ExportSellReportData(
            PeriodType period,
            string filePhysicalPath,
            DateTime startDate,
            DateTime endDate,
            string storeID,
            out string message,
            out string message1,
            out string fileNameLinkText
            )
        {
            SaleReportBuilder reportBuilder = new SaleReportBuilder();
            _table = reportBuilder.GetReportData(
                  period,
                  startDate,
                  endDate,
                  storeID);

            string filePathName = "Export/SaleReport_from_" +
                startDate.Month + "_" + startDate.Day + "_" + startDate.Year + "_to_" +
                endDate.Month + "_" + endDate.Day + "_" + endDate.Year + ".csv";

            string filePhysicalPathName = filePhysicalPath + filePathName;

            if (_table.Rows.Count > 0)
            {
                StreamWriter writer = new StreamWriter( filePhysicalPathName, false );
                writer.WriteLine( GenerateHeader() );

                string details;
                foreach (DataRow row in _table.Rows)
                {
                    details = GenerateDetail( row );
                    writer.WriteLine( details );
                }
                writer.WriteLine( GenerateFooter() );
                writer.Dispose();
            }
            if (_table.Rows.Count == 0)
            {
                message = "No data to export.";
                message1 = "";
                fileNameLinkText = "";
                return "";
            }
            else
            {
                message = "Export file is successfully create.";
                message1 = "Download ";
                fileNameLinkText = Path.GetFileName( filePathName );
                return "../DownloadFile.aspx?FilePath=" + "../" + filePathName;
            }
        }
    }
}
