﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;
using Vevo.Domain;
using Vevo.Domain.EmailTemplates;
using Vevo.Domain.Marketing;
using Vevo.Domain.Orders;
using Vevo.Domain.Payments;
using Vevo.Domain.Products;
using Vevo.Domain.Returns;
using Vevo.Domain.Stores;
using Vevo.Shared.Utilities;
using Vevo.Shared.WebUI;
using Vevo.WebUI.Users;
using Vevo.Domain.Users;
using Vevo.Deluxe.Domain;
using Vevo.Deluxe.Domain.GiftRegistry;
using Vevo.Deluxe.Domain.BundlePromotion;

namespace Vevo.WebUI
{
    public static class EmailTemplateTextVariable
    {
        public const string TemplateFolder = "ContentTemplates/";
        public const string EmailTempalteVariableList = "EmailTemplateVariableList.xml";

        #region Private

        private static bool IsNameExist( string emailName, string emailTemplateName )
        {
            if (emailTemplateName.Contains( "|" ))
            {
                string[] emailTemplateNameList = emailTemplateName.Split( '|' );
                foreach (string name in emailTemplateNameList)
                {
                    if (emailName == name)
                    {
                        return true;
                    }
                }
            }
            else
            {
                if (emailName == emailTemplateName)
                {
                    return true;
                }
            }

            return false;
        }

        private static string GetRegExCurrencyValue( string value )
        {
            if (value.Contains( "$" ))
            {
                return "$" + value;
            }
            return value;
        }

        private static EmailTemplateDetail GetEmailTemplateDetail( string emailTemplateName, string storeID )
        {
            string emailID = DataAccessContext.EmailTemplateDetailRepository.GetIDByNameAndStoreID(
                            StoreContext.Culture,
                            emailTemplateName,
                            storeID );

            return DataAccessContext.EmailTemplateDetailRepository.GetOne( StoreContext.Culture, emailID );
        }

        private static EmailTemplateDetail GetEmailTemplateDetail( string emailTemplateName )
        {
            return GetEmailTemplateDetail( emailTemplateName, new StoreRetriever().GetCurrentStoreID() );
        }
        private static string ReplaceAffiliateAgreementTextVariable( string content )
        {
            return content;
        }

        private static string ReplaceAffiliateApproveTextVariable( string content, string username, string email )
        {
            Uri uri = new Uri( UrlPath.StorefrontUrl );
            content = content.Replace( "[SiteName]", NamedConfig.SiteName );
            content = content.Replace( "[AffiliateLink]",
                UrlPath.StorefrontUrl + "AffiliateLogin.aspx?ReturnUrl=" + HttpUtility.UrlEncode( uri.PathAndQuery + "AffiliateDashboard.aspx" ) );
            content = content.Replace( "[UserName]", username );
            content = content.Replace( "[Email]", email );

            return content;
        }

        private static string ReplaceAffiliatePaymentTextVariable( string content, string paidDate, string amount, string orderID )
        {
            content = content.Replace( "[PaidDate]", paidDate );
            content = content.Replace( "[Amount]", amount );
            content = content.Replace( "[OrderID]", orderID );

            return content;
        }

        private static string ReplaceAffiliateSubscribeTextVariable(
            string content,
            string affiliateViewUrl,
            string username,
            string email )
        {
            content = content.Replace( "[SiteName]", NamedConfig.SiteName );
            content = content.Replace( "[AffiliateViewUrl]", affiliateViewUrl );
            content = content.Replace( "[UserName]", username );
            content = content.Replace( "[Email]", email );

            return content;
        }

        private static string ReplaceConfirmPasswordTextVariable( string content, string username, string passwordLink )
        {
            content = content.Replace( "[SiteName]", NamedConfig.SiteName );
            content = content.Replace( "[UserName]", username );
            content = content.Replace( "[ResetPasswordLink]", passwordLink );

            return content;
        }

        private static string ReplaceResetPasswordTextVariable( string content, string username, string password )
        {
            content = content.Replace( "[SiteName]", NamedConfig.SiteName );
            content = content.Replace( "[UserName]", username );
            content = content.Replace( "[Password]", password );

            return content;
        }

        private static string ReplaceNewCustomerRegistrationTextVariable( string content, string username, string email )
        {
            content = content.Replace( "[SiteName]", NamedConfig.SiteName );
            content = content.Replace( "[UserName]", username );
            content = content.Replace( "[Email]", email );

            return content;
        }

        private static string ReplaceCustomerApprovalTextVariable(
            string content,
            string customerViewUrl,
            string username,
            string email )
        {
            content = content.Replace( "[SiteName]", NamedConfig.SiteName );
            content = content.Replace( "[CustomerViewUrl]", customerViewUrl );
            content = content.Replace( "[UserName]", username );
            content = content.Replace( "[Email]", email );

            return content;
        }

        private static string ReplaceCustomerApproveTextVariable( string content, string username, string email )
        {
            content = content.Replace( "[SiteName]", NamedConfig.SiteName );
            content = content.Replace( "[LoginLink]", UrlPath.StorefrontUrl + "UserLogin.aspx" );
            content = content.Replace( "[UserName]", username );
            content = content.Replace( "[Email]", email );

            return content;
        }

        private static string ReplaceDownloadFileLinkTextVariable( string content, string downloadLink )
        {
            content = content.Replace( "[DownloadLinks]", downloadLink );

            return content;
        }

        private static string ReplaceFedExTrackingTextVariable(
            string content,
            string siteName,
            string orderID,
            string trackingNumber,
            string trackingUrl )
        {
            content = content.Replace( "[SiteName]", siteName );
            content = content.Replace( "[OrderNo]", orderID );
            content = content.Replace( "[TrackingNumber]", trackingNumber );
            content = content.Replace( "[TrackUrl]", trackingUrl );

            return content;
        }

        private static string ReplaceUpsTrackingTextVariable(
            string content,
            string siteName,
            string orderID,
            string trackingNumber,
            string trackingUrl )
        {
            content = content.Replace( "[SiteName]", siteName );
            content = content.Replace( "[OrderNo]", orderID );
            content = content.Replace( "[TrackingNumber]", trackingNumber );
            content = content.Replace( "[TrackUrl]", trackingUrl );

            return content;
        }

        private static string ReplaceUspsTrackingTextVariable(
            string content,
            string siteName,
            string orderID,
            string trackingNumber,
            string trackingUrl )
        {
            content = content.Replace( "[SiteName]", siteName );
            content = content.Replace( "[OrderNo]", orderID );
            content = content.Replace( "[TrackingNumber]", trackingNumber );
            content = content.Replace( "[TrackUrl]", trackingUrl );

            return content;
        }

        private static string ReplaceOtherTrackingTextVariable(
            string content,
            string siteName,
            string orderID,
            string trackingNumber,
            string trackingUrl )
        {
            content = content.Replace( "[SiteName]", siteName );
            content = content.Replace( "[OrderNo]", orderID );
            content = content.Replace( "[TrackingNumber]", trackingNumber );
            content = content.Replace( "[TrackUrl]", trackingUrl );

            return content;
        }

        private static string ReplaceGiftCertificateCodeTextVariable(
            string content,
            string siteName,
            string orderID,
            StringBuilder url )
        {
            content = content.Replace( "[SiteName]", siteName );
            content = content.Replace( "[OrderNo]", orderID );
            content = content.Replace( "[Url]", url.ToString() );

            return content;
        }

        private static string ReplaceGiftCertificateLayoutTextVariable(
            string content,
            GiftCertificate giftCertificate,
            Order orders,
            string fromuser,
            string amount,
            string expireDate )
        {
            content = content.Replace( "[HeadCompany]",
            DataAccessContext.Configurations.GetValue( CultureUtilities.StoreCultureID, "CompanyName", orders.StoreID ) );
            content = content.Replace( "[HeadAddress]",
                DataAccessContext.Configurations.GetValue( CultureUtilities.StoreCultureID, "CompanyAddress", orders.StoreID ) );
            content = content.Replace( "[HeadCity]",
                DataAccessContext.Configurations.GetValue( CultureUtilities.StoreCultureID, "CompanyCity", orders.StoreID ) );
            content = content.Replace( "[HeadState]",
                DataAccessContext.Configurations.GetValue( CultureUtilities.StoreCultureID, "CompanyState", orders.StoreID ) );
            content = content.Replace( "[HeadCountry]",
                DataAccessContext.Configurations.GetValue( CultureUtilities.StoreCultureID, "CompanyCountry", orders.StoreID ) );
            content = content.Replace( "[HeadZipCode]",
                DataAccessContext.Configurations.GetValue( CultureUtilities.StoreCultureID, "CompanyZip", orders.StoreID ) );
            content = content.Replace( "[FROM]", fromuser );
            content = content.Replace( "[TO]", giftCertificate.Recipient );
            content = content.Replace( "[NOTE]", giftCertificate.PersonalNote );
            content = content.Replace( "[CODE]", giftCertificate.GiftCertificateCode.ToUpper() );
            content = content.Replace( "[AMOUNT]", amount );
            content = content.Replace( "[EXPIREDATE]", expireDate );

            return content;
        }

        private static string ReplaceGiftRegistryInvitationTextVariable( string content, GiftRegistry giftRegistry, string giftRegistryLink )
        {
            content = content.Replace( "[Name]", giftRegistry.ShippingAddress.FirstName + " " + giftRegistry.ShippingAddress.LastName );
            content = content.Replace( "[EventName]", giftRegistry.EventName );
            content = content.Replace( "[EventDate]", giftRegistry.EventDate.ToString( "D" ) );
            content = content.Replace( "[GiftRegistryLink]", giftRegistryLink );
            content = content.Replace( "[StoreName]", NamedConfig.SiteName );

            return content;
        }

        private static string ReplaceGiftRegistryNewOrderTextVariable( string content, Order order )
        {
            if (content.Contains( "[OrderDetails]" ))
            {
                StringBuilder emailMessage = new StringBuilder();

                emailMessage.Append( "Order Number: " );
                emailMessage.Append( order.OrderID );//[OrderNumber]
                emailMessage.Append( "\n\n" );

                emailMessage.Append( "Buyer Name: " + order.Billing.FirstName + " " + order.Billing.LastName );//[OrderFirstName] [OrderLastName]
                emailMessage.Append( "<br/><br/>" );
                emailMessage.Append( "*****  Items in this order  *****" );
                emailMessage.Append( "<br/><br/>" );

                IList<OrderItem> orderItems = DataAccessContext.OrderItemRepository.GetByOrderID( order.OrderID );

                for (int rowIndex = 0; rowIndex < orderItems.Count; rowIndex++)
                {
                    emailMessage.Append( "No: " ); 
                    emailMessage.Append( rowIndex + 1 );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( "Name: " );
                    emailMessage.Append( orderItems[rowIndex].GetPlainTextName() );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( "Quantity: " );
                    emailMessage.Append( orderItems[rowIndex].Quantity );
                    emailMessage.Append( "<br/><br/>" );
                }

                emailMessage.Append( "***************************" );
                emailMessage.Append( "<br/>" );

                content = content.Replace( "[OrderDetails]", emailMessage.ToString() );
            }

            content = content.Replace( "[OrderNo]", order.OrderID );
            content = content.Replace( "[OrderFirstName]", order.Shipping.FirstName );
            content = content.Replace( "[OrderLastName]", order.Shipping.LastName );

            if (content.Contains( "[OrderItems" ))
            {
                StringBuilder emailMessage = new StringBuilder();
                string[] textValue = Regex.Split( content, "\\[OrderItems\\s+ItemNumberText=\"([^\\[]+)\",\\s+NameText=\"([^\\[]+)\",\\s+QuantityText=\"([^\\[]+)\"\\]" );

                IList<OrderItem> orderItems = DataAccessContext.OrderItemRepository.GetByOrderID( order.OrderID );

                for (int rowIndex = 0; rowIndex < orderItems.Count; rowIndex++)
                {//[OrderItems]
                    emailMessage.Append( textValue[1] ); // [OrderItems ItemNumberText="", NameText="", QunatityText=""]
                    emailMessage.Append( rowIndex + 1 );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( textValue[2] );
                    emailMessage.Append( orderItems[rowIndex].GetPlainTextName() );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( textValue[3] );
                    emailMessage.Append( orderItems[rowIndex].Quantity );
                    emailMessage.Append( "<br/><br/>" );
                }

                content = Regex.Replace( content, "\\[OrderItems\\s+ItemNumberText=\"([^\\[]+)\",\\s+NameText=\"([^\\[]+)\",\\s+QuantityText=\"([^\\[]+)\"\\]", emailMessage.ToString() );
            }
            else
            {
                content = Regex.Replace( content, "\\[OrderItems\\s+ItemNumberText=\"([^\\[]+)\",\\s+NameText=\"([^\\[]+)\",\\s+QuantityText=\"([^\\[]+)\"\\]", String.Empty );
            }

            return content;
        }

        private static string PopulateInvoiceAddress( Order order, Culture culture )
        {
            Store store = DataAccessContext.StoreRepository.GetOne( order.StoreID );

            return String.Format(
                "{0}<br />{1}&nbsp;{2}&nbsp;{3}&nbsp;{4}<br />" +
                "Phone&nbsp;:&nbsp;{5}&nbsp;<br />Fax&nbsp;:&nbsp;{6}<br />Email&nbsp;:&nbsp;{7}",
                DataAccessContext.Configurations.GetValue( culture.CultureID, "CompanyAddress", order.StoreID ),
                DataAccessContext.Configurations.GetValue( culture.CultureID, "CompanyCity", order.StoreID ),
                DataAccessContext.Configurations.GetValue( culture.CultureID, "CompanyState", order.StoreID ),
                DataAccessContext.Configurations.GetValue( culture.CultureID, "CompanyCountry", order.StoreID ),
                DataAccessContext.Configurations.GetValue( "CompanyZip", store ),
                DataAccessContext.Configurations.GetValue( "CompanyPhone", store ),
                DataAccessContext.Configurations.GetValue( "CompanyFax", store ),
                DataAccessContext.Configurations.GetValue( "CompanyEmail", store ) );
        }

        private static string PopulateOrderItemList( Order order, Currency currency )
        {
            IList<OrderItem> orderItemList = DataAccessContext.OrderItemRepository.GetByOrderID( order.OrderID );

            string result = String.Empty;
            for (int rowIndex = 0; rowIndex < orderItemList.Count; rowIndex++)
            {
                OrderItem item = orderItemList[rowIndex];
                decimal subtotal = ConvertUtilities.ToDecimal( item.UnitPrice * item.Quantity );

                result = String.Format(
                    "{5}<tr >" +
                    "<td align=\"center\">" +
                    "{0}</td>" +
                    "<td align=\"left\">" +
                    "{1}</td>" +
                    "<td align=\"right\">" +
                    "{2}</td>" +
                    "<td align=\"right\">" +
                    "{3}</td>" +
                    "<td align=\"right\">" +
                    "{4}</td>" +
                    "</tr>",
                    item.ProductID,
                    item.Name,
                    item.Quantity,
                    currency.FormatPrice( item.UnitPrice ),
                    currency.FormatPrice( subtotal ),
                    result );
            }

            return result;
        }

        private static string ReplaceInvoiceTextVariable( string content, Order order, Culture culture, Currency currency )
        {
            string address = PopulateInvoiceAddress( order, culture );

            string orderItemListText = PopulateOrderItemList( order, currency );


            content = content.Replace( "[HeadCompany]", DataAccessContext.Configurations.GetValue( culture.CultureID, "CompanyName", order.StoreID ) );
            content = content.Replace( "[HeadAddress]", address );
            content = content.Replace( "[OrderID]", order.OrderID );

            content = content.Replace( "[Date]", DateTime.Today.ToShortDateString() );
            content = content.Replace( "[UserName]", order.UserName );
            content = content.Replace( "[FirstName]", order.Billing.FirstName );
            content = content.Replace( "[LastName]", order.Billing.LastName );
            content = content.Replace( "[Company]", order.Billing.Company );
            content = content.Replace( "[Address1]", order.Billing.Address1 );
            content = content.Replace( "[Address2]", order.Billing.Address2 );
            content = content.Replace( "[City]", order.Billing.City );
            content = content.Replace( "[State]", order.Billing.State );
            content = content.Replace( "[Country]", order.Billing.Country );
            content = content.Replace( "[Zip]", order.Billing.Zip );
            content = content.Replace( "[Phone]", order.Billing.Phone );
            content = content.Replace( "[Fax]", order.Billing.Fax );
            content = content.Replace( "[Email]", order.Email );
            content = content.Replace( "[ShippingFirstName]", order.Shipping.FirstName );
            content = content.Replace( "[ShippingLastName]", order.Shipping.LastName );
            content = content.Replace( "[ShippingCompany]", order.Shipping.Company );
            content = content.Replace( "[ShippingAddress1]", order.Shipping.Address1 );
            content = content.Replace( "[ShippingAddress2]", order.Shipping.Address2 );
            content = content.Replace( "[ShippingCity]", order.Shipping.City );
            content = content.Replace( "[ShippingState]", order.Shipping.State );
            content = content.Replace( "[ShippingCountry]", order.Shipping.Country );
            content = content.Replace( "[ShippingZip]", order.Shipping.Zip );
            content = content.Replace( "[ShippingPhone]", order.Shipping.Phone );
            content = content.Replace( "[ShippingFax]", order.Shipping.Fax );
            content = content.Replace( "[CustomerNote]", order.CustomerComments );
            content = content.Replace( "[InvoiceNotes]", order.InvoiceNotes );
            content = content.Replace( "[SubTotal]", currency.FormatPrice( order.Subtotal ) );
            content = content.Replace( "[Tax]", currency.FormatPrice( order.Tax ) );
            content = content.Replace( "[Discount]", currency.FormatPrice( order.CouponDiscount ) );
            content = content.Replace( "[ShippingCost]", currency.FormatPrice( order.ShippingCost ) );
            content = content.Replace( "[Total]", currency.FormatPrice(
                (order.Subtotal + order.Tax + order.ShippingCost) - order.CouponDiscount ) );

            content = content.Replace( "[OrderItem]", orderItemListText );

            return content;
        }

        private static string ReplaceNewAffiliateRegistrationTextVariable( string content, string username, string email )
        {
            content = content.Replace( "[SiteName]", NamedConfig.SiteName );
            content = content.Replace( "[UserName]", username );
            content = content.Replace( "[Email]", email );

            return content;
        }

        private static string ReplaceNewsLetterTextVariable( string content, string email, string emailHash, string storeID )
        {
            StoreRetriever storeRetriever = new StoreRetriever();
            string urlUnsubscribe = storeRetriever.GetStorefrontUrlByStoreID( storeID ) + "/Newsletter.aspx";
            string emailLink = urlUnsubscribe + "?Act=Unsubscribe&Email=" + email + "&Key=" + emailHash;

            content = Regex.Replace( content, "\\[UnsubscribeLink\\s+Text=\"([^\\[]+)\"\\]", "<a href=" + emailLink + " target=_blank >$1</a>" );
            content = Regex.Replace( content, "\\[UnsubscribeLink\\s+Text=&quot;([^\\[]+)&quot;\\]", "<a href=" + emailLink + " target=_blank >$1</a>" );
            content = Regex.Replace( content, "\\[UnsubscribeUrl\\]", emailLink );

            return content;
        }

        private static string ReplaceConfirmationSubscribeVariable( string content, string email, string comfirmLink )
        {
            content = content.Replace( "[SiteName]", NamedConfig.SiteName );
            content = content.Replace( "[Email]", email );
            content = content.Replace( "[ConfirmationLink]", comfirmLink );
            return content;
        }

        private static string FormatProductName( string name )
        {
            return name.Replace( "<br>", ", " );
        }

        private static bool IsGiftCertificateEnabled()
        {
            return DataAccessContext.Configurations.GetBoolValue( "GiftCertificateEnabled" );
        }

        private static bool IsHandlingFeeEnabled()
        {
            return DataAccessContext.Configurations.GetBoolValue( "HandlingFeeEnabled" );
        }

        private static string GetPaymentTypeText( string paymentMethodName )
        {
            PaymentOption paymentOption = DataAccessContext.PaymentOptionRepository.GetOne(
                StoreContext.Culture, PaymentOption.GetSimpleName( paymentMethodName ) );
            PaymentMethod paymentMethod = paymentOption.CreatePaymentMethod();

            if (paymentOption.ShownAsCreditCard)
                return "Credit Card";
            else
                return paymentMethodName;
        }

        private static string ReplaceOrderConfirmationTextVariable( string content, Order order )
        {
            Store store = DataAccessContext.StoreRepository.GetOne( order.StoreID );
            string siteName = DataAccessContext.Configurations.GetValue( CultureUtilities.StoreCultureID, "SiteName", store );

            if (content.Contains( "[OrderDetails]" ))
            {
                StringBuilder emailMessage = new StringBuilder();

                emailMessage.Append( "Order Number: " );
                emailMessage.Append( order.OrderID );
                emailMessage.Append( "<br/><br/>" );

                emailMessage.Append( "Name: " + order.Shipping.FirstName + " " + order.Shipping.LastName );
                emailMessage.Append( "<br/><br/>" );
                emailMessage.Append( "*****  Items in this order  *****" );
                emailMessage.Append( "<br/><br/>" );

                IList<OrderItem> list = DataAccessContext.OrderItemRepository.GetByOrderID( order.OrderID );
                int i = 1;
                for (int itemIndex = 0; itemIndex < list.Count; itemIndex++)
                {
                    emailMessage.Append( "No: " );
                    emailMessage.Append( i++ );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( "Name: " );
                    emailMessage.Append( FormatProductName( list[itemIndex].Name.Replace( "<br/>", Environment.NewLine ) ) );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( "Sku: " );
                    emailMessage.Append( list[itemIndex].Sku );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( "Quantity: " );
                    emailMessage.Append( list[itemIndex].Quantity.ToString() );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( "Unit Price: " );
                    emailMessage.Append( StoreContext.Currency.FormatPrice( list[itemIndex].UnitPrice ) );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( "Total: " );
                    decimal total = ConvertUtilities.ToDecimal( list[itemIndex].Quantity * list[itemIndex].UnitPrice );

                    emailMessage.Append( StoreContext.Currency.FormatPrice( total ) );

                    emailMessage.Append( "<br/><br/>" );
                }

                emailMessage.Append( "***************************" );
                emailMessage.Append( "<br/>" );

                string emailShipping = order.Email;

                emailMessage.Append( "Product Cost: " );
                emailMessage.Append( StoreContext.Currency.FormatPrice( order.Subtotal ) );
                emailMessage.Append( "<br/>" );
                emailMessage.Append( "Discount: " );
                emailMessage.Append( StoreContext.Currency.FormatPrice( order.CouponDiscount ) );
                emailMessage.Append( "<br/>" );
                if (DataAccessContext.Configurations.GetBoolValue( "PointSystemEnabled", StoreContext.CurrentStore ))
                {
                    emailMessage.Append( "Point Discount: " );
                    emailMessage.Append( StoreContext.Currency.FormatPrice( order.RedeemPrice ) );
                    emailMessage.Append( "<br/>" );
                }
                emailMessage.Append( "Tax: " );
                emailMessage.Append( StoreContext.Currency.FormatPrice( order.Tax ) );
                emailMessage.Append( "<br/>" );
                emailMessage.Append( "Shipping Cost: " );
                emailMessage.Append( StoreContext.Currency.FormatPrice( order.ShippingCost ) );
                emailMessage.Append( "<br/>" );
                if (IsHandlingFeeEnabled())
                {
                    emailMessage.Append( "Handling Fee: " );
                    emailMessage.Append( StoreContext.Currency.FormatPrice( order.HandlingFee ) );
                    emailMessage.Append( "<br/>" );
                }
                if (IsGiftCertificateEnabled())
                {
                    emailMessage.Append( "Gift Certificate: " );
                    emailMessage.Append( StoreContext.Currency.FormatPrice( order.GiftCertificate ) );
                    emailMessage.Append( "<br/>" );
                }
                emailMessage.Append( "--------------------------------------" );
                emailMessage.Append( "<br/>" );
                emailMessage.Append( "Total: " );
                emailMessage.Append( StoreContext.Currency.FormatPrice( order.Total ) );
                emailMessage.Append( "<br/>" );
                emailMessage.Append( "--------------------------------------" );
                emailMessage.Append( "<br/>" );

                emailMessage.Append( "<br/>" );
                emailMessage.Append( "Payment Type: " );
                emailMessage.Append( GetPaymentTypeText( order.PaymentMethod ) );
                emailMessage.Append( "<br/>" );

                emailMessage.Append( "Order Date: " );
                emailMessage.Append( order.OrderDate );
                emailMessage.Append( "<br/>" );

                if (DataAccessContext.Configurations.GetBoolValue( "PointSystemEnabled", StoreContext.CurrentStore )
                    && order.RedeemPoint == 0)
                {
                    emailMessage.Append( "Reward Point Earned: " );
                    emailMessage.Append( order.PointEarned );
                    emailMessage.Append( "<br/>" );
                }

                emailMessage.Append( "Special Request: " );
                emailMessage.Append( order.CustomerComments );
                emailMessage.Append( "<br/><br/>" );

                if (DataAccessContext.Configurations.GetBoolValue( "ShippingAddressMode" ) && order.ShowShippingAddress)
                {
                    emailMessage.Append( "*****  This order will ship to  *****" );
                    emailMessage.Append( "<br/>" );

                    emailMessage.Append( "FirstName: " );
                    emailMessage.Append( order.Shipping.FirstName );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( "LastName: " );
                    emailMessage.Append( order.Shipping.LastName );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( "Company: " );
                    emailMessage.Append( order.Shipping.Company );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( "Address: " );
                    emailMessage.Append( order.Shipping.Address1 );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( order.Shipping.Address2 );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( "City: " );
                    emailMessage.Append( order.Shipping.City );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( "State: " );
                    emailMessage.Append( order.Shipping.State );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( "Zip: " );
                    emailMessage.Append( order.Shipping.Zip );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( "Country: " );
                    emailMessage.Append( AddressUtilities.GetCountryNameByCountryCode( order.Shipping.Country ) );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( "Email: " );
                    emailMessage.Append( emailShipping );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( "Phone: " );
                    emailMessage.Append( order.Shipping.Phone );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( "Fax: " );
                    emailMessage.Append( order.Shipping.Fax );
                    emailMessage.Append( "<br/>" );
                }

                content = content.Replace( "[OrderDetails]", emailMessage.ToString() );
            }

            content = content.Replace( "[OrderNo]", order.OrderID );
            content = content.Replace( "[OrderFirstName]", order.Shipping.FirstName );
            content = content.Replace( "[OrderLastName]", order.Shipping.LastName );
            if (content.Contains( "[OrderItems" ))
            {
                StringBuilder emailMessage = new StringBuilder();
                string[] textValue = Regex.Split( content, "\\[OrderItems\\s+NoText=\"([^\\[]+)\",\\s+NameText=\"([^\\[]+)\",\\s+SkuText=\"([^\\[]+)\",\\s+QuantityText=\"([^\\[]+)\",\\s+UnitPriceText=\"([^\\[]+)\",\\s+Total=\"([^\\[]+)\"\\]" );

                IList<OrderItem> list = DataAccessContext.OrderItemRepository.GetByOrderID( order.OrderID );
                int i = 1;
                for (int itemIndex = 0; itemIndex < list.Count; itemIndex++)
                {
                    emailMessage.Append( textValue[1] );
                    emailMessage.Append( i++ );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( textValue[2] );
                    emailMessage.Append( FormatProductName( list[itemIndex].Name.Replace( "<br/>", Environment.NewLine ) ) );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( textValue[3] );
                    emailMessage.Append( list[itemIndex].Sku );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( textValue[4] );
                    emailMessage.Append( list[itemIndex].Quantity.ToString() );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( textValue[5] );
                    emailMessage.Append( GetRegExCurrencyValue( StoreContext.Currency.FormatPrice( list[itemIndex].UnitPrice ) ) );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( textValue[6] );
                    decimal total = ConvertUtilities.ToDecimal( list[itemIndex].Quantity * list[itemIndex].UnitPrice );

                    emailMessage.Append( GetRegExCurrencyValue( StoreContext.Currency.FormatPrice( total ) ) );

                    emailMessage.Append( "<br/><br/>" );
                }

                content = Regex.Replace( content, "\\[OrderItems\\s+NoText=\"([^\\[]+)\",\\s+NameText=\"([^\\[]+)\",\\s+SkuText=\"([^\\[]+)\",\\s+QuantityText=\"([^\\[]+)\",\\s+UnitPriceText=\"([^\\[]+)\",\\s+Total=\"([^\\[]+)\"\\]", emailMessage.ToString() );
            }
            else
            {
                content = Regex.Replace( content, "\\[OrderItems\\s+NoText=\"([^\\[]+)\",\\s+NameText=\"([^\\[]+)\",\\s+SkuText=\"([^\\[]+)\",\\s+QuantityText=\"([^\\[]+)\",\\s+UnitPriceText=\"([^\\[]+)\",\\s+Total=\"([^\\[]+)\"\\]", String.Empty );
            }

            content = content.Replace( "[SiteName]", siteName );
            content = content.Replace( "[OrderNo]", order.OrderID );
            content = content.Replace( "[OrderFirstName]", order.Shipping.FirstName );
            content = content.Replace( "[OrderLastName]", order.Shipping.LastName );
            content = content.Replace( "[OrderProductCost]", StoreContext.Currency.FormatPrice( order.Subtotal ) );
            content = content.Replace( "[OrderDiscount]", StoreContext.Currency.FormatPrice( order.CouponDiscount ) );
            content = content.Replace( "[OrderTax]", StoreContext.Currency.FormatPrice( order.Tax ) );
            content = content.Replace( "[OrderShippingCost]", StoreContext.Currency.FormatPrice( order.ShippingCost ) );
            content = content.Replace( "[OrderTotal]", StoreContext.Currency.FormatPrice( order.Total ) );
            content = content.Replace( "[OrderPaymentType]", GetPaymentTypeText( order.PaymentMethod ) );
            content = content.Replace( "[OrderDate]", order.OrderDate.ToString() );
            content = content.Replace( "[OrderSpecialRequest]", order.CustomerComments );

            if (DataAccessContext.Configurations.GetBoolValue( "PointSystemEnabled", StoreContext.CurrentStore ))
            {
                if (content.Contains( "[PointDiscount" ))
                {
                    StringBuilder emailMessage = new StringBuilder();
                    string[] textValue = Regex.Split( content, "\\[PointDiscount\\s+Text=\"([^\\[]+)\"\\]" );
                    if (textValue.Length > 1)
                    {
                        emailMessage.Append( textValue[1] );
                        emailMessage.Append( GetRegExCurrencyValue( StoreContext.Currency.FormatPrice( order.RedeemPrice ) ) );

                        content = Regex.Replace( content, "\\[PointDiscount\\s+Text=\"([^\\[]+)\"\\]", emailMessage.ToString() );
                    }
                    else
                    {
                        content = Regex.Replace( content, "\\[PointDiscount\\s+Text=\"([^\\[]+)\"\\]", String.Empty );
                    }
                }
                else
                {
                    content = Regex.Replace( content, "\\[PointDiscount\\s+Text=\"([^\\[]+)\"\\]", String.Empty );
                }

                if (content.Contains( "[OrderRewardPointEarned" ))
                {
                    StringBuilder emailMessage = new StringBuilder();
                    string[] textValue = Regex.Split( content, "\\[OrderRewardPointEarned\\s+Text=\"([^\\[]+)\"\\]" );
                    if (textValue.Length > 1)
                    {
                        emailMessage.Append( textValue[1] );
                        emailMessage.Append( order.PointEarned );

                        content = Regex.Replace( content, "\\[OrderRewardPointEarned\\s+Text=\"([^\\[]+)\"\\]", emailMessage.ToString() );
                    }
                    else
                    {
                        content = Regex.Replace( content, "\\[OrderRewardPointEarned\\s+Text=\"([^\\[]+)\"\\]", String.Empty );
                    }
                }
                else
                {
                    content = Regex.Replace( content, "\\[OrderRewardPointEarned\\s+Text=\"([^\\[]+)\"\\]", String.Empty );
                }
            }
            else
            {
                content = Regex.Replace( content, "\\[PointDiscount\\s+Text=\"([^\\[]+)\"\\]", String.Empty );
                content = Regex.Replace( content, "\\[OrderRewardPointEarned\\s+Text=\"([^\\[]+)\"\\]", String.Empty );
            }

            if (IsHandlingFeeEnabled() && content.Contains( "[OrderHandlingFee" ))
            {
                StringBuilder emailMessage = new StringBuilder();
                string[] textValue = Regex.Split( content, "\\[OrderHandlingFee\\s+Text=\"([^\\[]+)\"\\]" );
                if (textValue.Length > 1)
                {
                    emailMessage.Append( textValue[1] );
                    emailMessage.Append( GetRegExCurrencyValue( StoreContext.Currency.FormatPrice( order.HandlingFee ) ) );

                    content = Regex.Replace( content, "\\[OrderHandlingFee\\s+Text=\"([^\\[]+)\"\\]", emailMessage.ToString() );
                }
                else
                {
                    content = Regex.Replace( content, "\\[OrderHandlingFee\\s+Text=\"([^\\[]+)\"\\]", String.Empty );
                }
            }
            else
            {
                content = Regex.Replace( content, "\\[OrderHandlingFee\\s+Text=\"([^\\[]+)\"\\]", String.Empty );
            }

            if (IsGiftCertificateEnabled() && content.Contains( "[OrderGiftCertificate" ))
            {
                StringBuilder emailMessage = new StringBuilder();
                string[] textValue = Regex.Split( content, "\\[OrderGiftCertificate\\s+Text=\"([^\\[]+)\"\\]" );
                if (textValue.Length > 1)
                {
                    emailMessage.Append( textValue[1] );
                    emailMessage.Append( GetRegExCurrencyValue( StoreContext.Currency.FormatPrice( order.GiftCertificate ) ) );
                    content = Regex.Replace( content, "\\[OrderGiftCertificate\\s+Text=\"([^\\[]+)\"\\]", emailMessage.ToString() );
                }
                else
                {
                    content = Regex.Replace( content, "\\[OrderGiftCertificate\\s+Text=\"([^\\[]+)\"\\]", String.Empty );
                }
            }
            else
            {
                content = Regex.Replace( content, "\\[OrderGiftCertificate\\s+Text=\"([^\\[]+)\"\\]", String.Empty );
            }

            if (DataAccessContext.Configurations.GetBoolValue( "ShippingAddressMode" ) &&
                order.ShowShippingAddress &&
                content.Contains( "[ShippingAddress" ))
            {
                StringBuilder emailMessage = new StringBuilder();
                string[] textValue = Regex.Split( content, "\\[ShippingAddress\\s+HeadText=\"([^\\[]+)\",\\s+FirstNameText=\"([^\\[]+)\",\\s+LastNameText=\"([^\\[]+)\",\\s+CompanyText=\"([^\\[]+)\",\\s+AddressText=\"([^\\[]+)\",\\s+CityText=\"([^\\[]+)\",\\s+StateText=\"([^\\[]+)\",\\s+ZipText=\"([^\\[]+)\",\\s+CountryText=\"([^\\[]+)\",\\s+EmailText=\"([^\\[]+)\",\\s+PhoneText=\"([^\\[]+)\",\\s+FaxText=\"([^\\[]+)\"\\]" );
                if (textValue.Length > 1)
                {
                    emailMessage.Append( textValue[1] );
                    emailMessage.Append( "<br/>" );

                    emailMessage.Append( textValue[2] ); 
                    emailMessage.Append( order.Shipping.FirstName );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( textValue[3] ); 
                    emailMessage.Append( order.Shipping.LastName );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( textValue[4] ); 
                    emailMessage.Append( order.Shipping.Company );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( textValue[5] ); 
                    emailMessage.Append( order.Shipping.Address1 );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( order.Shipping.Address2 );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( textValue[6] );
                    emailMessage.Append( order.Shipping.City );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( textValue[7] ); 
                    emailMessage.Append( order.Shipping.State );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( textValue[8] ); 
                    emailMessage.Append( order.Shipping.Zip );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( textValue[9] ); 
                    emailMessage.Append( AddressUtilities.GetCountryNameByCountryCode( order.Shipping.Country ) );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( textValue[10] ); 
                    emailMessage.Append( order.Email );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( textValue[11] ); 
                    emailMessage.Append( order.Shipping.Phone );
                    emailMessage.Append( "<br/>" );
                    emailMessage.Append( textValue[12] ); 
                    emailMessage.Append( order.Shipping.Fax );
                    emailMessage.Append( "<br/>" ); ;

                    content = Regex.Replace( content, "\\[ShippingAddress\\s+HeadText=\"([^\\[]+)\",\\s+FirstNameText=\"([^\\[]+)\",\\s+LastNameText=\"([^\\[]+)\",\\s+CompanyText=\"([^\\[]+)\",\\s+AddressText=\"([^\\[]+)\",\\s+CityText=\"([^\\[]+)\",\\s+StateText=\"([^\\[]+)\",\\s+ZipText=\"([^\\[]+)\",\\s+CountryText=\"([^\\[]+)\",\\s+EmailText=\"([^\\[]+)\",\\s+PhoneText=\"([^\\[]+)\",\\s+FaxText=\"([^\\[]+)\"\\]", emailMessage.ToString() );
                }
                else
                {
                    content = Regex.Replace( content, "\\[ShippingAddress\\s+HeadText=\"([^\\[]+)\",\\s+FirstNameText=\"([^\\[]+)\",\\s+LastNameText=\"([^\\[]+)\",\\s+CompanyText=\"([^\\[]+)\",\\s+AddressText=\"([^\\[]+)\",\\s+CityText=\"([^\\[]+)\",\\s+StateText=\"([^\\[]+)\",\\s+ZipText=\"([^\\[]+)\",\\s+CountryText=\"([^\\[]+)\",\\s+EmailText=\"([^\\[]+)\",\\s+PhoneText=\"([^\\[]+)\",\\s+FaxText=\"([^\\[]+)\"\\]", String.Empty );
                }
            }
            else
            {
                content = Regex.Replace( content, "\\[ShippingAddress\\s+HeadText=\"([^\\[]+)\",\\s+FirstNameText=\"([^\\[]+)\",\\s+LastNameText=\"([^\\[]+)\",\\s+CompanyText=\"([^\\[]+)\",\\s+AddressText=\"([^\\[]+)\",\\s+CityText=\"([^\\[]+)\",\\s+StateText=\"([^\\[]+)\",\\s+ZipText=\"([^\\[]+)\",\\s+CountryText=\"([^\\[]+)\",\\s+EmailText=\"([^\\[]+)\",\\s+PhoneText=\"([^\\[]+)\",\\s+FaxText=\"([^\\[]+)\"\\]", String.Empty );
            }

            return content;
        }

        private static string GetImage( string imagePath )
        {
            if (String.IsNullOrEmpty( imagePath ))
            {
                return "<Img src=\"" + UrlPath.StorefrontUrl + "/Images/Products/Thumbnail/DefaultNoImage.gif\">";
            }
            else
            {
                return "<Img src=\"" + UrlPath.StorefrontUrl + imagePath + "\">";
            }
        }

        private static string ReplaceTellFriendTextVariable( string content, Product product )
        {
            ProductPrice productPrice = product.GetProductPrice( new StoreRetriever().GetCurrentStoreID() );

            string imgUrl = GetImage( product.ImageSecondary );


            string textLink = UrlPath.StorefrontUrl + product.UrlName + "-details.aspx";
            string productSalePrice;
            string productRetailPrice;

            if (product.IsCustomPrice || product.IsCallForPrice || productPrice.RetailPrice <= 0)
            {
                productRetailPrice = "-";
                productSalePrice = "-";
                content = content.Replace( "<span style='text-decoration: line-through;'>[RetailPrice]</span>", "[RetailPrice]" );
            }
            else
            {
                productRetailPrice = StoreContext.Currency.FormatPrice( productPrice.RetailPrice );
                productSalePrice = StoreContext.Currency.FormatPrice( productPrice.Price );
            }

            content = content.Replace( "[Name]", product.Name );

            if (DataAccessContext.Configurations.GetBoolValue( "PriceRequireLogin" ))
            {
                content = content.Replace( "[RetailPrice]", "" );
                content = content.Replace( "[Price]", "" );
            }
            else
            {
                content = content.Replace( "[RetailPrice]", productRetailPrice );
                content = content.Replace( "[Price]", productSalePrice );
            }

            content = content.Replace( "[ShortDescription]", product.ShortDescription );
            content = content.Replace( "[ImageSecondary]", imgUrl );

            content = Regex.Replace( content, "\\[TellFriendTextLink\\s+Text=\"([^\\[]+)\"\\]", "<a href=" + textLink + " target=_blank >$1</a>" );
            content = Regex.Replace( content, "\\[TellFriendTextLink\\s+Text=&quot;([^\\[]+)&quot;\\]", "<a href=" + textLink + " target=_blank >$1</a>" );

            return content;
        }

        private static string ReplaceTellFriendPromotionTextVariable( string content, PromotionGroup group )
        {
            IList<PromotionGroupSubGroup> subGroupList = DataAccessContextDeluxe.PromotionGroupRepository.GetPromotionGroupSubGroup( group.PromotionGroupID );

            string imgUrl = GetImage( group.ImageFile );

            string textLink = UrlPath.StorefrontUrl + group.UrlName + "-promo.aspx";
            string productSalePrice;

            productSalePrice = StoreContext.Currency.FormatPrice( group.Price );
            content = content.Replace( "[Name]", group.Name );
            content = content.Replace( "[Price]", productSalePrice );
            content = content.Replace( "[ShortDescription]", group.Description );

            content = content.Replace( "[ImageSecondary]", imgUrl );

            content = Regex.Replace( content, "\\[TellFriendTextLink\\s+Text=\"([^\\[]+)\"\\]", "<a href=" + textLink + " target=_blank >$1</a>" );
            content = Regex.Replace( content, "\\[TellFriendTextLink\\s+Text=&quot;([^\\[]+)&quot;\\]", "<a href=" + textLink + " target=_blank >$1</a>" );

            return content;
        }
        private static string ReplaceRMARequisitionTextVariable( string content, Rma rma )
        {
            content = content.Replace( "[SiteName]", NamedConfig.SiteName );
            content = content.Replace( "[RmaViewUrl]", UrlPath.StorefrontUrl + DataAccessContext.Configurations.GetValue( "AdminAdvancedFolder" ) +
            "/Default.aspx#RmaEdit,RmaID=" + rma.RmaID );
            content = content.Replace( "[UserName]", rma.GetCustomer.UserName );
            content = content.Replace( "[Email]", rma.GetCustomer.Email );

            return content;
        }

        private static string ReplaceRMAApprovalTextVariable( string content, Rma rma )
        {
            content = content.Replace( "[SiteName]", NamedConfig.SiteName );

            string result = String.Format(
                    "<table>" +
                    "<tr >" +
                    "   <td align=\"center\">RMA ID</td>" +
                    "   <td align=\"left\">{0}</td>" +
                    "</tr>" +
                    "<tr >" +
                    "   <td align=\"center\">Product</td>" +
                    "   <td align=\"left\">{1}</td>" +
                    "</tr>" +
                    "<tr >" +
                    "   <td align=\"center\">Quantity</td>" +
                    "   <td align=\"left\">{2}</td>" +
                    "</tr>" +
                    "<tr >" +
                    "   <td align=\"center\">Date</td>" +
                    "   <td align=\"left\">{3}</td>" +
                    "</tr>" +
                    "<tr >" +
                    "   <td align=\"center\">Note</td>" +
                    "   <td align=\"left\">{4}</td>" +
                    "</tr>" +
                    "</table>",
                    rma.RmaID,
                    rma.ProductName,
                    rma.Quantity,
                    rma.RequestDate,
                    rma.RmaNote );

            content = content.Replace( "[RmaDetails]", result );
            content = content.Replace( "[RMAID]", rma.RmaID );
            content = content.Replace( "[RMAProduct]", rma.ProductName );
            content = content.Replace( "[RMAQuantity]", ConvertUtilities.ToString( rma.Quantity ) );
            content = content.Replace( "[RMADate]", ConvertUtilities.ToString( rma.RequestDate ) );
            content = content.Replace( "[RMANote]", rma.RmaNote );

            return content;
        }

        private static string ReplaceRMARejectedTextVariable( string content, Rma rma )
        {
            content = content.Replace( "[SiteName]", NamedConfig.SiteName );

            string result = String.Format(
                    "<table>" +
                    "<tr >" +
                    "   <td align=\"center\">RMA ID</td>" +
                    "   <td align=\"left\">{0}</td>" +
                    "</tr>" +
                    "<tr >" +
                    "   <td align=\"center\">Product</td>" +
                    "   <td align=\"left\">{1}</td>" +
                    "</tr>" +
                    "<tr >" +
                    "   <td align=\"center\">Quantity</td>" +
                    "   <td align=\"left\">{2}</td>" +
                    "</tr>" +
                    "<tr >" +
                    "   <td align=\"center\">Date</td>" +
                    "   <td align=\"left\">{3}</td>" +
                    "</tr>" +
                    "<tr >" +
                    "   <td align=\"center\">Note</td>" +
                    "   <td align=\"left\">{4}</td>" +
                    "</tr>" +
                    "</table>",
                    rma.RmaID,
                    rma.ProductName,
                    rma.Quantity,
                    rma.RequestDate,
                    rma.RmaNote );

            content = content.Replace( "[RmaDetails]", result );
            content = content.Replace( "[RMAID]", rma.RmaID );
            content = content.Replace( "[RMAProduct]", rma.ProductName );
            content = content.Replace( "[RMAQuantity]", ConvertUtilities.ToString( rma.Quantity ) );
            content = content.Replace( "[RMADate]", ConvertUtilities.ToString( rma.RequestDate ) );
            content = content.Replace( "[RMANote]", rma.RmaNote );

            return content;
        }

        private static string ReplaceSubscriptionContentTextVariable( string content, string orderID, string subscriptionLevel )
        {
            Store store = DataAccessContext.StoreRepository.GetOne( DataAccessContext.OrderRepository.GetOne( orderID ).StoreID );
            content = content.Replace( "[OrderNo]", orderID );
            content = content.Replace( "[SubscriptionLevel]", subscriptionLevel );
            content = content.Replace( "[SiteName]", store.StoreName );
            content = content.Replace( "[SiteUrl]", store.UrlName );
            return content;
        }

        #endregion

        #region Public

        public static DataTable GetVariableList( string emailName )
        {
            DataSet ds = new DataSet();
            string pathFile = HttpContext.Current.Server.MapPath( "~/ContentTemplates/EmailTemplateVariableList.xml" );
            if (File.Exists( pathFile ))
            {
                ds.ReadXml( pathFile );

                ArrayList rowsTobeDeleted = new ArrayList();
                foreach (DataRow row in ds.Tables[0].Rows)
                {
                    string emailTemplateName = row["EmailName"].ToString();

                    if (!IsNameExist( emailName, emailTemplateName ))
                    {
                        rowsTobeDeleted.Add( row );
                    }
                }

                for (int i = 0; i < rowsTobeDeleted.Count; i++)
                {
                    ds.Tables[0].Rows.Remove( (DataRow) rowsTobeDeleted[i] );
                }
            }
            else
                throw new Exception( "File Not Found : " + pathFile );

            return ds.Tables[0];
        }
        public static void ReplaceAffiliateAgreementText( out string bodyText )
        {
            string emailTemplateID = DataAccessContext.EmailTemplateDetailRepository.GetIDByNameAndStoreID( StoreContext.Culture, "Affiliate Agreement", StoreContext.CurrentStore.StoreID );
            EmailTemplateDetail email = DataAccessContext.EmailTemplateDetailRepository.GetOne( StoreContext.Culture, emailTemplateID );

            bodyText = ReplaceAffiliateAgreementTextVariable( email.Body );
        }
        public static void ReplaceAffiliateApproveText( string username, string email, out string subjectText, out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Affiliate Approve" );

            subjectText = ReplaceAffiliateApproveTextVariable( emailDetail.Subject, username, email );
            bodyText = ReplaceAffiliateApproveTextVariable( emailDetail.Body, username, email );
        }

        public static void ReplaceAffiliatePaymentText( string paidDate, string amount, string orderID, out string subjectText, out string bodyText )
        {
            Order order = DataAccessContext.OrderRepository.GetOne( orderID );

            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Affiliate Payment Commission", order.StoreID );

            subjectText = ReplaceAffiliatePaymentTextVariable( emailDetail.Subject, paidDate, amount, orderID );
            bodyText = ReplaceAffiliatePaymentTextVariable( emailDetail.Body, paidDate, amount, orderID );
        }

        public static void ReplaceAffiliateSubscribeText(
            string affiliateViewUrl,
            string username,
            string email,
            out string subjectText,
            out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Affiliate Subscribe" );

            subjectText = ReplaceAffiliateSubscribeTextVariable( emailDetail.Subject, affiliateViewUrl, username, email );
            bodyText = ReplaceAffiliateSubscribeTextVariable( emailDetail.Body, affiliateViewUrl, username, email );
        }

        public static void ReplaceConfirmationPasswordText( string username, string passwordLink, out string subjectText, out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Reset Password Confirmation" );

            subjectText = ReplaceConfirmPasswordTextVariable( emailDetail.Subject, username, passwordLink );
            bodyText = ReplaceConfirmPasswordTextVariable( emailDetail.Body, username, passwordLink );
        }

        public static void ReplaceResetPasswordText( string username, string password, out string subjectText, out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Reset Password" );

            subjectText = ReplaceResetPasswordTextVariable( emailDetail.Subject, username, password );
            bodyText = ReplaceResetPasswordTextVariable( emailDetail.Body, username, password );
        }

        public static bool ReplaceDownloadFileLinkText( string downloadLink, string storeID, out string subjectText, out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "e-Good Download Link", storeID );
            subjectText = "";
            bodyText = "";

            if (emailDetail.Subject.Contains( "[DownloadLinks]" ) || emailDetail.Body.Contains( "[DownloadLinks]" ))
            {
                subjectText = ReplaceDownloadFileLinkTextVariable( emailDetail.Subject, downloadLink );
                bodyText = ReplaceDownloadFileLinkTextVariable( emailDetail.Body, downloadLink );

                return true;
            }

            return false;
        }

        public static void ReplaceNewCustomerRegistrationText( string username, string email, out string subjectText, out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "New Customer Registration" );

            subjectText = ReplaceNewCustomerRegistrationTextVariable( emailDetail.Subject, username, email );
            bodyText = ReplaceNewCustomerRegistrationTextVariable( emailDetail.Body, username, email );
        }

        public static void NewCustomerApproveRegistrationText(
            string customerViewUrl,
            string username,
            string email,
            out string subjectText,
            out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Customer Approval" );

            subjectText = ReplaceCustomerApprovalTextVariable( emailDetail.Subject, customerViewUrl, username, email );
            bodyText = ReplaceCustomerApprovalTextVariable( emailDetail.Body, customerViewUrl, username, email );
        }

        public static void ReplaceCustomerApproveText( string username, string email, out string subjectText, out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Customer Approval Confirmation" );

            subjectText = ReplaceCustomerApproveTextVariable( emailDetail.Subject, username, email );
            bodyText = ReplaceCustomerApproveTextVariable( emailDetail.Body, username, email );
        }

        public static void ReplaceTrackingText(
            string siteName,
            Order order,
            string trackingUrl,
            out string subjectText,
            out string bodyText )
        {
            if (order.TrackingMethod.ToLower() == "ups")
            {
                EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Ups Tracking", order.StoreID );

                subjectText = ReplaceUpsTrackingTextVariable( emailDetail.Subject, siteName, order.OrderID, order.TrackingNumber, trackingUrl );
                bodyText = ReplaceUpsTrackingTextVariable( emailDetail.Body, siteName, order.OrderID, order.TrackingNumber, trackingUrl );
            }
            else if (order.TrackingMethod.ToLower() == "fedex")
            {
                EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "FedEx Tracking", order.StoreID );

                subjectText = ReplaceFedExTrackingTextVariable( emailDetail.Subject, siteName, order.OrderID, order.TrackingNumber, trackingUrl );
                bodyText = ReplaceFedExTrackingTextVariable( emailDetail.Body, siteName, order.OrderID, order.TrackingNumber, trackingUrl );
            }
            else if (order.TrackingMethod.ToLower() == "usps")
            {
                EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Ups Tracking", order.StoreID );

                subjectText = ReplaceUspsTrackingTextVariable( emailDetail.Subject, siteName, order.OrderID, order.TrackingNumber, trackingUrl );
                bodyText = ReplaceUspsTrackingTextVariable( emailDetail.Body, siteName, order.OrderID, order.TrackingNumber, trackingUrl );
            }
            else
            {
                EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Other Tracking" );

                subjectText = ReplaceOtherTrackingTextVariable( emailDetail.Subject, siteName, order.OrderID, order.TrackingNumber, trackingUrl );
                bodyText = ReplaceOtherTrackingTextVariable( emailDetail.Body, siteName, order.OrderID, order.TrackingNumber, trackingUrl );
            }
        }

        public static void ReplaceGiftCertificateCodeText(
            string siteName,
            Order order,
            StringBuilder url,
            out string subjectText,
            out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Gift Certificate Code", order.StoreID );

            subjectText = ReplaceGiftCertificateCodeTextVariable( emailDetail.Subject, siteName, order.OrderID, url );
            bodyText = ReplaceGiftCertificateCodeTextVariable( emailDetail.Body, siteName, order.OrderID, url );
        }

        public static void ReplaceGiftCertificateLayoutText(
            GiftCertificate giftCertificate,
            Order orders,
            string fromuser,
            string amount,
            string expireDate,
            out string subjectText,
            out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Gift Certificate Layout", orders.StoreID );

            subjectText = ReplaceGiftCertificateLayoutTextVariable( emailDetail.Subject, giftCertificate, orders, fromuser, amount, expireDate );
            bodyText = ReplaceGiftCertificateLayoutTextVariable( emailDetail.Body, giftCertificate, orders, fromuser, amount, expireDate );
        }

        public static void ReplaceGiftRegistryInvitationText(
            GiftRegistry giftRegistry,
            string giftRegistryLink,
            out string subjectText,
            out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Gift Registry Invitation" );

            subjectText = ReplaceGiftRegistryInvitationTextVariable( emailDetail.Subject, giftRegistry, giftRegistryLink );
            bodyText = ReplaceGiftRegistryInvitationTextVariable( emailDetail.Body, giftRegistry, giftRegistryLink );
        }

        public static void ReplaceGiftRegistryNewOrderText( Order order, out string subjectText, out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Gift Registry Purchase Notification" );

            subjectText = ReplaceGiftRegistryNewOrderTextVariable( emailDetail.Subject, order );
            bodyText = ReplaceGiftRegistryNewOrderTextVariable( emailDetail.Body, order );
        }

        public static void ReplaceInvoiceText( Order order, Culture culture, Currency currency, out  string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Advanced Invoice Layout", order.StoreID );

            bodyText = ReplaceInvoiceTextVariable( emailDetail.Body, order, culture, currency );
        }

        public static void ReplaceNewAffiliateRegistrationText( string username, string email, out string subjectText, out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "New Affiliate Registration" );

            subjectText = ReplaceNewAffiliateRegistrationTextVariable( emailDetail.Subject, username, email );
            bodyText = ReplaceNewAffiliateRegistrationTextVariable( emailDetail.Body, username, email );
        }

        public static string ReplaceNewsLetterText( string emailBody, string email, string emailHash, string storeID )
        {
            return ReplaceNewsLetterTextVariable( emailBody, email, emailHash, storeID );
        }

        public static string ReplaceNewsLetterText( string email, string emailHash, string storeID )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Newsletter Layout", storeID );

            return ReplaceNewsLetterTextVariable( emailDetail.Body, email, emailHash, storeID );
        }

        public static void ReplaceConfirmationSubscribeText( string email, string comfirmLink, out string subjectText, out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Subscribe Newsletter Confirmation" );

            subjectText = ReplaceConfirmationSubscribeVariable( emailDetail.Subject, email, comfirmLink );
            bodyText = ReplaceConfirmationSubscribeVariable( emailDetail.Body, email, comfirmLink );
        }

        public static void ReplaceOrderConfirmationText( Order order, out string subjectText, out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Order Confirmation" );

            subjectText = ReplaceOrderConfirmationTextVariable( emailDetail.Subject, order );
            bodyText = ReplaceOrderConfirmationTextVariable( emailDetail.Body, order );
        }

        public static void ReplaceTellFriendText(
            Product product,
            out string subjectText,
            out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Tell A Friend" );

            subjectText = ReplaceTellFriendTextVariable(
                emailDetail.Subject,
                product );

            bodyText = ReplaceTellFriendTextVariable(
                emailDetail.Body,
                product );
        }

        public static void ReplaceTellFriendPromotionText(
            PromotionGroup group,
            out string subjectText,
            out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Promotion Tell A Friend" );

            subjectText = ReplaceTellFriendPromotionTextVariable(
                emailDetail.Subject,
                group );

            bodyText = ReplaceTellFriendPromotionTextVariable(
                emailDetail.Body,
                group );
        }

        public static void ReplaceRMARequisitionText( Rma rma, out string subjectText, out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "RMA Requisition" );

            subjectText = ReplaceRMARequisitionTextVariable( emailDetail.Subject, rma );
            bodyText = ReplaceRMARequisitionTextVariable( emailDetail.Body, rma );
        }

        public static void ReplaceRMAApprovalText( Rma rma, out string subjectText, out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "RMA Approval" );

            subjectText = ReplaceRMAApprovalTextVariable( emailDetail.Subject, rma );
            bodyText = ReplaceRMAApprovalTextVariable( emailDetail.Body, rma );
        }

        public static void ReplaceRMARejectedText( Rma rma, out string subjectText, out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "RMA Rejected" );

            subjectText = ReplaceRMARejectedTextVariable( emailDetail.Subject, rma );
            bodyText = ReplaceRMARejectedTextVariable( emailDetail.Body, rma );
        }

        public static bool ReplaceSubscriptionContentLinkText(
            string orderID, string subscriptionLevel, string storeID, out string subjectText, out string bodyText )
        {
            EmailTemplateDetail emailDetail = GetEmailTemplateDetail( "Subscription Content Link", storeID );
            subjectText = "";
            bodyText = "";

            if (emailDetail.Subject.Contains( "[SubscriptionLevel]" ) || emailDetail.Body.Contains( "[SubscriptionLevel]" ))
            {
                subjectText = ReplaceSubscriptionContentTextVariable( emailDetail.Subject, orderID, subscriptionLevel );
                bodyText = ReplaceSubscriptionContentTextVariable( emailDetail.Body, orderID, subscriptionLevel );
                return true;
            }

            return false;

        }
        #endregion
    }
}
