using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI.WebControls;
using Vevo.Domain;
using Vevo.Domain.Products;
using Vevo.WebUI.International;

namespace Vevo.WebUI.Products
{
    public class BaseCategoryMenuNavListUserControl : BaseLanguageUserControl
    {
        private string[] GetCategoryListWithLeaf( string categoryID )
        {
            IList<string> catList = new List<string>();
            IList<string> categoryIDs = DataAccessContext.CategoryRepository.GetLeafFromCategoryID( categoryID, catList );

            List<string> categoryCollection = new List<string>();
            foreach (string categoryItem in categoryIDs)
            {
                categoryCollection.Add( categoryItem );
            }

            return categoryCollection.ToArray();
        }

        public int GetCountProductByCategoryID( string categoryID )
        {
            return DataAccessContext.ProductRepository.GetProductCountByCategoryID( GetCategoryListWithLeaf( categoryID ) );
        }

        private string[] GetDepartmentListWithLeaf( string departmentID )
        {
            IList<string> catList = new List<string>();
            IList<string> departmentIDs = DataAccessContext.DepartmentRepository.GetLeafFromDepartmentID( departmentID, catList );

            List<string> departmentCollection = new List<string>();
            foreach (string departmentItem in departmentIDs)
            {
                departmentCollection.Add( departmentItem );
            }

            return departmentCollection.ToArray();
        }

        public int GetCountProductByDepartmentID( string departmentID )
        {
            return DataAccessContext.ProductRepository.GetProductCountByDepartmentID( GetDepartmentListWithLeaf( departmentID ) );
        }

        private void PopulateCategories( int nodeDepth, MenuItem item, string parentCategoryID )
        {
            IList<Category> list = DataAccessContext.CategoryRepository.GetByParentID(
                StoreContext.Culture,
                parentCategoryID,
                "SortOrder",
                BoolFilter.ShowTrue );

            foreach (Category category in list)
            {
                item.ChildItems.Add( CreateMenuItemWithChildren( nodeDepth, category ) );
            }
        }

        protected MenuItem CreateMenuItemWithChildren( int nodeDepth, Category category )
        {
            MenuItem newItem = new MenuItem();
            newItem.Text = category.Name;
            newItem.Value = category.CategoryID;

            if (DataAccessContext.CategoryRepository.IsCategoryIDNotLeaf( category.CategoryID ))
            {
                newItem.NavigateUrl = UrlManager.GetCategoryUrl( category.CategoryID, category.UrlName );

                if (nodeDepth < MaxNode)
                    PopulateCategories( nodeDepth + 1, newItem, category.CategoryID );
            }
            else
            {
                newItem.NavigateUrl = UrlManager.GetCategoryUrl( category.CategoryID, category.UrlName );
            }
            return newItem;
        }

        protected void SetSelectedCategoryMenuItem( MenuItem menuItem, string categoryID, string categoryName )
        {
            if (menuItem.Value == categoryID || menuItem.Text == categoryName)
            {
                if (menuItem.Value == CategoryID || menuItem.Text == CategoryName)
                    menuItem.Selected = true;
                return;
            }
            else
            {
                foreach (MenuItem mi in menuItem.ChildItems)
                {
                    if (mi.Value == categoryID || mi.Text == categoryName)
                    {
                        if (mi.Value == CategoryID || mi.Text == CategoryName)
                            mi.Selected = true;
                        SetSelectedCategoryMenuItem( menuItem, mi.Parent.Value, mi.Parent.Text );
                        return;
                    }
                    SetSelectedCategoryMenuItem( mi, categoryID, categoryName );
                }
            }
        }

        private void PopulateDepartments( int nodeDepth, MenuItem item, string parentDepartmentID )
        {
            IList<Department> list = DataAccessContext.DepartmentRepository.GetByParentID(
                StoreContext.Culture,
                parentDepartmentID,
                "SortOrder",
                BoolFilter.ShowTrue );

            foreach (Department department in list)
            {
                item.ChildItems.Add( CreateMenuItemWithChildren( nodeDepth, department ) );
            }
        }

        protected MenuItem CreateMenuItemWithChildren( int nodeDepth, Department department )
        {
            MenuItem newItem = new MenuItem();
            newItem.Text = department.Name;
            newItem.Value = department.DepartmentID;

            if (DataAccessContext.DepartmentRepository.IsDepartmentIDNotLeaf( department.DepartmentID ))
            {
                newItem.NavigateUrl = UrlManager.GetDepartmentUrl( department.DepartmentID, department.UrlName );

                if (nodeDepth < MaxNode)
                    PopulateDepartments( nodeDepth + 1, newItem, department.DepartmentID );
            }
            else
            {
                newItem.NavigateUrl = UrlManager.GetDepartmentUrl( department.DepartmentID, department.UrlName );
            }
            return newItem;
        }

        protected MenuItem CreateMenuItemWithChildren( Manufacturer manufacturer )
        {
            MenuItem newItem = new MenuItem();
            newItem.Text = manufacturer.Name;
            newItem.Value = manufacturer.ManufacturerID;

            newItem.NavigateUrl = UrlManager.GetManufacturerUrl( manufacturer.ManufacturerID, manufacturer.UrlName );

            return newItem;
        }

        public int MaxNode
        {
            get
            {
                if (ViewState["MaxNode"] == null)
                    return 0;
                return Convert.ToInt32( ViewState["MaxNode"] );
            }
            set
            {
                ViewState["MaxNode"] = value;
            }
        }

        public string CategoryID
        {
            get
            {
                if (ViewState["CategoryID"] == null)
                    return "0";
                else
                    return ViewState["CategoryID"].ToString();
            }
            set
            {
                ViewState["CategoryID"] = value;
            }
        }

        public string CategoryName
        {
            get
            {
                if (ViewState["CategoryName"] == null)
                    return String.Empty;
                else
                    return ViewState["CategoryName"].ToString();
            }
            set
            {
                ViewState["CategoryName"] = value;
            }
        }

        public string DepartmentID
        {
            get
            {
                if (ViewState["DepartmentID"] == null)
                    return "0";
                else
                    return ViewState["DepartmentID"].ToString();
            }
            set
            {
                ViewState["DepartmentID"] = value;
            }
        }

        public string DepartmentName
        {
            get
            {
                if (ViewState["DepartmentName"] == null)
                    return String.Empty;
                else
                    return ViewState["DepartmentName"].ToString();
            }
            set
            {
                ViewState["DepartmentName"] = value;
            }
        }
    }
}
